/*
Copyright 2014 Paul Soulsby www.soulsbysynths.com
    This file is part of Atmegatron.

    Atmegadrum is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Atmegadrum is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Atmegadrum.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
USE_GITHUB_USERNAME=soulsbysynths
*/

#define MEM_PATCHSIZE 32
#define CTRL_FILT 0
#define CTRL_Q 1
#define CTRL_ENV 2
#define CTRL_LFO 3
#define CTRL_AMP 4
#define CTRL_FX 5

#define POLL_HARD 1
#define POLL_MIDI 1
#define POLL_TEST 0
#define MASTER_BEATTICK 500   //bpm, 500 = 120bpm i.e. 1 beat = 500ms
#define MASTER_SAMPS 16


unsigned long master_tick;
volatile uint16_t master_index[16];
volatile signed int master_sum;
volatile unsigned char master_output;
volatile unsigned long master_ocr1 = 2273;
unsigned long master_sf = 7040;   //sample frequency, ocr1 derived from this
const int master_minocr1 = 900;

void setup() {
  
  Init_Master();
  Init_Register();
  Init_Hardware();
  

  if (POLL_MIDI == 1){
	  MIDI_Init();
  }
  else{
	  Serial.begin(9600);
  }

  Hardware_LED_StartFlash(0,5);
}


void loop(){
  if (POLL_MIDI==1){
	  MIDI_Poll();
  }
  if (POLL_HARD==1){
    Hardware_Poll();
  }
  master_tick = millis();

  Seq_Process();
  LFO_CalcVal();  //LFO always needs calc, never stops
  Pitch_Process();
  Master_Let_SampleFreq();
}

void Master_Let_SampleFreq(){
  unsigned long tmp;
  if (Pitch_Get_PitchChanged()==true){
    master_sf = Pitch_Get_FreqCalc();//*WAVE_LEN;
    tmp = F_CPU/master_sf;
    if (tmp>65535){
      tmp=65535;
    }
    if (tmp<master_minocr1){
      tmp=master_minocr1;
    }
    master_ocr1 = tmp;
  }
}
unsigned long Master_Get_SampleFreq(){
  return master_sf;
}
void Init_Master(){
  byte i;
  for(i=0;i<16;i++){
    master_index[i] = Wavestore_Get_Start(i+1)-1;
  }  
}
ISR(TIMER1_COMPA_vect) {
  master_sum =  (Wavestore_Get_Sample(master_index[0])>>Seq_Get_Amplitude(0)) + 
                (Wavestore_Get_Sample(master_index[1])>>Seq_Get_Amplitude(1)) + 
                (Wavestore_Get_Sample(master_index[2])>>Seq_Get_Amplitude(2)) +
                (Wavestore_Get_Sample(master_index[3])>>Seq_Get_Amplitude(3)) + 
                (Wavestore_Get_Sample(master_index[4])>>Seq_Get_Amplitude(4)) + 
                (Wavestore_Get_Sample(master_index[5])>>Seq_Get_Amplitude(5)) + 
                (Wavestore_Get_Sample(master_index[6])>>Seq_Get_Amplitude(6)) + 
                (Wavestore_Get_Sample(master_index[7])>>Seq_Get_Amplitude(7)) + 
                (Wavestore_Get_Sample(master_index[8])>>Seq_Get_Amplitude(8)) + 
                (Wavestore_Get_Sample(master_index[9])>>Seq_Get_Amplitude(9)) + 
                (Wavestore_Get_Sample(master_index[10])>>Seq_Get_Amplitude(10)) + 
                (Wavestore_Get_Sample(master_index[11])>>Seq_Get_Amplitude(11)) + 
                (Wavestore_Get_Sample(master_index[12])>>Seq_Get_Amplitude(12)) + 
                (Wavestore_Get_Sample(master_index[13])>>Seq_Get_Amplitude(13)) +
                (Wavestore_Get_Sample(master_index[14])>>Seq_Get_Amplitude(14)) + 
                (Wavestore_Get_Sample(master_index[15])>>Seq_Get_Amplitude(15));

  if (master_sum>127){
    master_output=255;
  }
  else if (master_sum<-128){
    master_output=0;
  }
  else{
    master_output = master_sum + 128;
  }

  OCR2B = master_output;
  if(master_index[0]<Wavestore_Get_Start(1)) master_index[0]++; 
  if(master_index[1]<Wavestore_Get_Start(2)) master_index[1]++; 
  if(master_index[2]<Wavestore_Get_Start(3)) master_index[2]++; 
  if(master_index[3]<Wavestore_Get_Start(4)) master_index[3]++; 
  if(master_index[4]<Wavestore_Get_Start(5)) master_index[4]++; 
  if(master_index[5]<Wavestore_Get_Start(6)) master_index[5]++; 
  if(master_index[6]<Wavestore_Get_Start(7)) master_index[6]++; 
  if(master_index[7]<Wavestore_Get_Start(8)) master_index[7]++; 
  if(master_index[8]<Wavestore_Get_Start(9)) master_index[8]++; 
  if(master_index[9]<Wavestore_Get_Start(10)) master_index[9]++; 
  if(master_index[10]<Wavestore_Get_Start(11)) master_index[10]++; 
  if(master_index[11]<Wavestore_Get_Start(12)) master_index[11]++; 
  if(master_index[12]<Wavestore_Get_Start(13)) master_index[12]++; 
  if(master_index[13]<Wavestore_Get_Start(14)) master_index[13]++; 
  if(master_index[14]<Wavestore_Get_Start(15)) master_index[14]++; 
  if(master_index[15]<Wavestore_Get_Start(16)) master_index[15]++; 
  OCR1A = master_ocr1;
} 







