/*
Copyright 2014 Paul Soulsby www.soulsbysynths.com
    This file is part of Atmegatron.

    Atmegadrum is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Atmegadrum is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Atmegadrum.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <SPI.h>              // We use this library, so it must be called here.
#include <MCP23S17.h>         // Here is the new class to make using the MCP23S17 easy.

//#define ENCODER_DO_NOT_USE_INTERRUPTS
#define ENCODER_USE_INTERRUPTS
#define ENCODER_OPTIMIZE_INTERRUPTS
#include <Encoder.h>

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#define DEBOUNCETICKS 100
#define LEDFLASHTICKS 50
#define LOCKOUT 1000

#define FLASHLONGTIME 300
#define FLASHSHORTTIME 150
#define MAXVAL 15

//PROG MEM
const unsigned int bg_select_store[16] PROGMEM = {1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768};
const unsigned int speed_store[16] PROGMEM =  {32767,16383,8191,6143,5460,4095,3071,2730,2047,1535,1023,767,511,383,255,127};                               

//hardware
MCP hard_bg_func(0);
MCP hard_bg_val(1);
MCP hard_bg_led(2);
unsigned int cur_bg_func=0;
unsigned int cur_bg_val=0;
unsigned int cur_bg_led=0;
Encoder hard_enc_func(5, 4);
Encoder hard_enc_val(2, 7);

//lets and gets (not all letable or getable at pres)
byte hard_func = 0;
byte hard_val = 0;
byte hard_anal[6] = {0,0,0,0,0,0};
boolean hard_shift = false;

unsigned long flashtick = 0;
boolean flashon = false;

//local variables
byte hard_curanalbank = 0;
byte hard_curanal = 0;
byte rawbuttons = B00000000;
byte dbbuttons = B00000000;
boolean curbankbut = false;
boolean curfuncbut = false;
boolean curvalbut = false;
byte lastctrl[6];
boolean movingctrl[6];
unsigned long stopmovingtick[6];
unsigned long butstarttick = 0;
unsigned long debouncecurtick = 0;
boolean finishedhold = false;

unsigned long ledflashnexttick[3];
boolean ledflashstate[3];
byte ledflashtimes[3];
byte ledflashcur[3];
boolean ledflashing[3];

void Init_Hardware(){
  byte i,j;
  long newfunc, newval;
  byte newanal;
  int analread;  
  //buttons
  DDRB = DDRB & B11111100;  //set digital 8-9 inputs, leave rest
  DDRD = DDRD & B10111111;
  
  hard_bg_func.pinMode(0x0000);    // Use word-write mode to Set all of the pins on outputchip to be outputs
  hard_bg_val.pinMode(0x0000);
  hard_bg_led.pinMode(0x0000);
  
  //set function bg to 0
  Hardware_Refresh_FuncBG();
  Hardware_Refresh_ShiftLED();
  
  Hardware_Refresh_ValBG();
  Hardware_Refresh_valLED();
  
  //set anal bank LED
  Hardware_Let_AnalBank(0);
 
  //Init analogs position
  for (i=0;i<=5;i++){
    analread = analogRead(i);
    newanal = analread >> 2;
    lastctrl[i] = newanal;
  }
  hard_curanal = 0;
  ADMUX = (1 << 6) | (hard_curanal & 0x07);
  // start the conversion
  sbi(ADCSRA, ADSC);
}


//*************POLL HARDWARE***********************
void Hardware_Poll(){
  long newfunc, newval;
	boolean newbankbut, newfuncbut, newvalbut;
  byte r, i;
  int newanal;
  byte  diff;
  unsigned long butcurtick;
  uint8_t low, high;
  
//  // READ FUNTCION   - doesn't use interrupt so try debounce to ignore 4 pulses.
  newfunc = hard_enc_func.read();
  if (master_tick > debouncecurtick && newfunc!=0) {
    if (newfunc > 2){
      if (hard_func==15){
        Hardware_Let_Function(0);
      }
      else{
        Hardware_Let_Function(hard_func+1);
      }
      hard_enc_func.write(0);
      debouncecurtick = master_tick + DEBOUNCETICKS;
    }
    else if (newfunc < -2) {
      if (hard_func==0){
        Hardware_Let_Function(15);
      }
      else{
        Hardware_Let_Function(hard_func-1);
      }
      hard_enc_func.write(0);    
      debouncecurtick = master_tick + DEBOUNCETICKS;
    }
  }
  else {
    hard_enc_func.write(0);
  }


//  //READ VALUE
  newval = hard_enc_val.read();
  if (newval!=0){
    if (newval >= 4){
      if (hard_val==MAXVAL){
        Hardware_Let_Value(0);
      }
      else if (hard_val<MAXVAL){
        Hardware_Let_Value(hard_val+1);
      }
      hard_enc_val.write(0);
    }
    else if (newval <= -4) {
      if (hard_val==0){
        Hardware_Let_Value(MAXVAL);
      }
      else if (hard_val>0){
        Hardware_Let_Value(hard_val-1);
      }
      hard_enc_val.write(0);
    }
  }

  rawbuttons = (~PINB & B00000011) | (~PIND & B01000000);
  if (master_tick > debouncecurtick) {
		//BUTTONS CHANGED
		if (rawbuttons!=dbbuttons){
			//ANAL BANK BUTTON
			if((rawbuttons & B01000000) != (dbbuttons & B01000000)){
				curbankbut = (rawbuttons & B01000000) >> 6;
				if (curbankbut==true){
					if (hard_curanalbank==0){
						Hardware_Let_AnalBank(1);
					}
					else{
						Hardware_Let_AnalBank(0);
					}
					butstarttick = master_tick;
					finishedhold = false;
				}
			}

			//READ L BUTTON
			if ((rawbuttons & B00000001) != (dbbuttons & B00000001)){
				curfuncbut = rawbuttons & B00000001;
				if (curfuncbut==true){
					butstarttick = master_tick;
					Hard_Let_Shift(true);
					finishedhold = false;
				}
				else {
					butcurtick = master_tick - butstarttick;
					if (butcurtick < 2000){     
						//DO LOAD HERE
						if(Seq_Get_Playing()==true){
							Memory_Let_LoadOnBeat(hard_func);
							Hardware_LED_StartFlash(0,253);
						}
						else{
							Memory_Load(hard_func);
						}
					}
					Hard_Let_Shift(false);
				}
			}
			//Read R BUTTON
			if ((rawbuttons & B00000010) != (dbbuttons & B00000010)){
				curvalbut = (rawbuttons & B00000010) >> 1;
				if (curvalbut==true){
					Seq_Let_Pattern(hard_func,hard_val,!Seq_Get_Pattern(hard_func,hard_val));
					Hardware_Refresh_valLED();
				}
			}
			dbbuttons = rawbuttons;			
			debouncecurtick = master_tick + DEBOUNCETICKS;
		}
		//NO CHANGE IN BUTTONS
		else {
			//ANAL BANK
			if (curbankbut==true){
				butcurtick = master_tick - butstarttick;
				if (butcurtick >= 2000 && finishedhold == false){
					finishedhold = true;
					Hardware_LED_StartFlash(2,7);
					Hardware_Let_AnalBank(0);
					Seq_Clear();
					Hardware_Refresh_valLED();
					Hardware_Refresh_ValBG();
				}				
			}
			//L BUTTON
			if (curfuncbut==true){
				butcurtick = master_tick - butstarttick;
				if (butcurtick >= 2000 && finishedhold == false){
					//DO SAVE HERE
					finishedhold = true;
					Memory_Save(hard_func);
					Hardware_LED_StartFlash(0,7);
				}
			}
		}
	}
	
  
  // READ 1 ANALOGUE
  //see if its time to lock pot
  if (master_tick >= stopmovingtick[hard_curanal]){
    movingctrl[hard_curanal] = false;
  }
  //now see if pot has moved
  if (bit_is_set(ADCSRA, ADSC)==false){
    low  = ADCL;
    high = ADCH;
    newanal = ((high << 8) | low) >> 2;
    if (newanal > lastctrl[hard_curanal]){
      diff = newanal-lastctrl[hard_curanal];
    }
    else{
      diff = lastctrl[hard_curanal]-newanal;
    }
    if (diff > 4){    //knob lockout sensitivity set here
      movingctrl[hard_curanal] = true;
      stopmovingtick[hard_curanal] = master_tick + LOCKOUT;  //lock after 5000ms
    }
    if (movingctrl[hard_curanal] == true){
      Hardware_Let_Analogue(hard_curanal, newanal);
      lastctrl[hard_curanal] = newanal;
    }  
  
    hard_curanal++;
    if (hard_curanal>5){
      hard_curanal = 0;
    }
    ADMUX = (1 << 6) | (hard_curanal & 0x07);
    // start the conversion
    sbi(ADCSRA, ADSC);
  }
 
  //CHECK FLASHING LEDS
  Hardware_LED_RefreshFlash();
  Hardware_Refresh_ValBG();

}


//**************ANALOGUE KNOBS*************************

void Hardware_Let_Analogue(byte index, byte value){ 
  hard_anal[index] = value;
  switch (index){
    case CTRL_FILT:      //Filter Cutoff
      Seq_Let_Tempo(map(value,0,255,60,255));
      break;
    case CTRL_Q:
			if(value==0){
				Pitch_Let_CurFreq(8000);
			}
			else{
				Pitch_Let_CurFreq(map(value,0,255,master_minocr1,16000));
			}
			break;
    case CTRL_ENV:
				Pitch_Let_LFOAmt(value);
      break;
    case CTRL_LFO:
        LFO_Let_Speed(pgm_read_word_near(&(speed_store[value>>4])));//select lfo speed  (stored in PROGMEM);
      break;
    case CTRL_AMP:
				Seq_Let_DelayBeats(hard_func,value>>5);
      break;
    case CTRL_FX:      //Post FX, was Master Gain
        Seq_Let_Volume(hard_func,7-(value>>5));
      break;
  }
}
byte Hardware_Get_Analogue(byte index){
  return hard_anal[index];
}

//**************FUNCTION KNOB*******************
void Hardware_Let_Function(byte newfunc){
  if (newfunc!=hard_func){  //don't allow floating positions, what to do???
    byte i;
    hard_func = newfunc;
    for(i=0;i<6;i++){
      movingctrl[i] = false;  //stops anal knobs from affecting different insts
    }
    Hardware_Refresh_FuncBG();
    Hardware_Refresh_ValBG();
    Hardware_Refresh_valLED();
  }
}

byte Hardware_Get_Function(){
  return hard_func;
}

void Hardware_Refresh_FuncBG(){
  hard_bg_func.digitalWrite(pgm_read_word_near(&(bg_select_store[hard_func])));
}

//*************VALUE KNOB***********************
void Hardware_Let_Value(byte newval){
  if (newval!=hard_val){   //don't allow floating positions, what to do??
    hard_val = newval;
    Hardware_Refresh_ValBG();
    Hardware_Refresh_valLED();
  }
}

byte Hardware_Get_Value(){
  return hard_val;
}
void Hardware_Refresh_valLED(){
  if (Seq_Get_Pattern(hard_func,hard_val)==true){
    hard_bg_led.digitalWrite(3,LOW);
    hard_bg_led.digitalWrite(2,HIGH);
  }
  else{
    hard_bg_led.digitalWrite(3,HIGH);
    hard_bg_led.digitalWrite(2,LOW);
  }
}

void Hardware_Refresh_ValBG(){
  unsigned int newvalbg=0; 
  unsigned int tmp=0;
  byte i;
  
  if(master_tick>=flashtick){
    flashon = !flashon;
    if(Seq_Get_Pattern(hard_func,hard_val)==false){
      if(flashon==true){
        flashtick +=  FLASHSHORTTIME;
      }
      else{
        flashtick +=  FLASHLONGTIME;
      }
    }
    else{
      if(flashon==true){
        flashtick +=  FLASHLONGTIME;
      }
      else{
        flashtick +=  FLASHSHORTTIME;
      }
    }
  }
  tmp = 1;
  for(i=0;i<16;i++){
    if((hard_val==i)&&(flashon==true)){
      newvalbg += tmp;
    }
    else if((hard_val!=i)&&(Seq_Get_Pattern(hard_func,i)!=false)){
      newvalbg += tmp;
    } 
    tmp *= 2; 
  }
  
  
  if (newvalbg!=cur_bg_val){
    cur_bg_val = newvalbg;
    hard_bg_val.digitalWrite(cur_bg_val);
  }
}

//*************ANAL BANK BUTTON***********************
void Hardware_Let_AnalBank(byte value){
  hard_curanalbank = value;
  Hardware_Refresh_AnalBankLED();
  Seq_Let_Playing((boolean)value,false);
}

byte Hardware_Get_AnalBank(){
  return hard_curanalbank;
}

void Hardware_Refresh_AnalBankLED(){
  
  if (hard_curanalbank==1){
    hard_bg_led.digitalWrite(5,LOW);
    hard_bg_led.digitalWrite(6,HIGH);
  }
  else{
    hard_bg_led.digitalWrite(5,HIGH);
    hard_bg_led.digitalWrite(6,LOW);
  }
}

//***********SHIFT BUTTON**************
void Hard_Let_Shift(boolean newshift){
	hard_shift = newshift;
	Hardware_Refresh_ShiftLED();

}
boolean Hard_Get_Shift(){
  return hard_shift;
}
void Hardware_Refresh_ShiftLED(){
	
	if (hard_shift==true){
		hard_bg_led.digitalWrite(1,LOW);
		hard_bg_led.digitalWrite(0,HIGH);
	}
	else{
		hard_bg_led.digitalWrite(1,HIGH);
		hard_bg_led.digitalWrite(0,LOW);
	}
}
//Flashing LEDS
void Hardware_LED_SetState(byte LEDnum, byte LEDstate){
  //0 = SHIFT left green
  //1 = SHIFT left red
  //2 = SHIFT right green
  //3 = SHIFT right red
  //4 = MIDI
  //5 = ANAL red  // I think turning red and green on together will result in both off, because both legs at 5V.
  //6 = ANAL green
  hard_bg_led.digitalWrite(LEDnum,LEDstate);
}
void Hardware_LED_StartFlash(byte LEDnum, byte FlashTimes){
  //0 = L SHIFT
  //1 = MIDI
	//2 = analbank
  if (LEDnum==0){
    hard_bg_led.digitalWrite(1,LOW);
    hard_bg_led.digitalWrite(0,HIGH); 
  }
  else if (LEDnum==1){
    hard_bg_led.digitalWrite(4,HIGH); 
  }
	else{
    hard_bg_led.digitalWrite(5,LOW);
    hard_bg_led.digitalWrite(6,HIGH);		
	}
  ledflashstate[LEDnum] = true;  
  ledflashtimes[LEDnum] = FlashTimes;
  ledflashcur[LEDnum] = 0;
  ledflashing[LEDnum] = true;
  ledflashnexttick[LEDnum] = master_tick + LEDFLASHTICKS;
}
void Hardware_LED_CancelFlash(byte LEDnum){
		ledflashtimes[LEDnum] = 0;
}
void Hardware_LED_RefreshFlash(){
  byte i;
  for(i=0;i<3;i++){
    if (ledflashing[i]==true){
      if (master_tick>=ledflashnexttick[i]){
        ledflashstate[i] = !ledflashstate[i];
        if(ledflashstate[i]==true){
          if (i==0){
            hard_bg_led.digitalWrite(1,LOW);
            hard_bg_led.digitalWrite(0,HIGH); 
          }
          else if (i==1){
            hard_bg_led.digitalWrite(4,HIGH);   
          }
					else {
						hard_bg_led.digitalWrite(5,LOW);
						hard_bg_led.digitalWrite(6,HIGH);						
					}
        }
        else{
          if (i==0){
            hard_bg_led.digitalWrite(1,HIGH);
            hard_bg_led.digitalWrite(0,LOW);  
          }
          else if (i==1){
            hard_bg_led.digitalWrite(4,LOW);
          }
					else {
						hard_bg_led.digitalWrite(5,HIGH);
						hard_bg_led.digitalWrite(6,LOW);
					}
        }
        ledflashcur[i]++;
        if(ledflashcur[i]>=ledflashtimes[i]){
          ledflashing[i] = false;
					if (i==0){
						Hardware_Refresh_ShiftLED();
					}
          if (i==2){
						Hardware_Refresh_AnalBankLED();
          }
        }
        else{
          ledflashnexttick[i] = master_tick + LEDFLASHTICKS;
        }
      }
    }
  }
}

