/*
Copyright 2014 Paul Soulsby www.soulsbysynths.com
    This file is part of Atmegatron.

    Atmegadrum is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Atmegadrum is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Atmegadrum.  If not, see <http://www.gnu.org/licenses/>.
*/

//**********Bipolar LFO.  16 waveforms, each 64 samples long.*************** 
#define LFO_LEN 64
#define PI 3.14159265359f

const char lfo_store[LFO_LEN] PROGMEM = {0,12,25,37,49,60,71,81,90,98,106,112,117,122,125,126,127,126,125,122,117,112,106,98,90,81,71,60,49,37,25,12,0,-12,-25,-37,-49,-60,-71,-81,-90,-98,-106,-112,-117,-122,-125,-126,-127,-126,-125,-122,-117,-112,-106,-98,-90,-81,-71,-60,-49,-37,-25,-12};
unsigned long  lfo_ticklen = 500;          //ticks per LFO cycle.  only used when MIDI clock not present.  default = 500 ticks, therefore 1 cycle per beat
unsigned long  lfo_clockticklen = 24;      //clock ticks per LFO cylcle.  only used when MIDI clock present.  24 ticks per beat, therefore 1 cycle per beat  
unsigned long lfo_starttick = 0;           //tick lfo starts oscillating at
unsigned long lfo_curtick = 0;             //cur tick of lfo

//lets and gets
unsigned int lfo_speed = 8191;             //speed of lfo.  not actual ticks, just a reference in a lookup table that is then divided to equal actual ticks.
char lfo_level = 0;                        //lfo output (-127 - 127)

//lets and gets
//LFO speed.  Stored as a lookup table of multipliers, not actual ticks
void LFO_Let_Speed(unsigned int newspeed){
  lfo_speed = newspeed;
  lfo_ticklen = ((unsigned long)lfo_speed+1) * MASTER_BEATTICK >> 11;    //Calculate lfo tick length.  This method is a way of calculating ticks without using slow (and memory consuming) floating point maths.  e.g. 1 cycle per beat = 8192 * 500 >> 13 = 500 
	if (lfo_ticklen<2){
    lfo_ticklen = 2;                                                     //Speed must be 2 or great otherwise divide by 0 in LFO_calcval
  }
  lfo_clockticklen = ((unsigned long)lfo_speed+1) * 24 >> 11;            //same as above, this time for when MIDI clock is present
	if (lfo_clockticklen<2){
        lfo_clockticklen = 2;                                            //Speed must be 2 or great otherwise divide by 0 in LFO_calcval
  }
}
unsigned int LFO_Get_Speed(){
  return lfo_speed;
}

//return current lfo level (-127 - 127)
char LFO_Get_Level(){
  return lfo_level;  
}

//LFO meat
//reset lfo start tick.  lfo is reset when first key is pressed
void LFO_Reset(){
  if (MIDI_Get_ClockPresent()==false){
    lfo_starttick = master_tick;
  }
}
//calculate the current output of the lfo
void LFO_CalcVal(){
  unsigned long pos; 
  int index;
  char lfoval;
  if (MIDI_Get_ClockPresent()==false){                                    //if midi clock not used:
    lfo_curtick = master_tick - lfo_starttick;                            //get ticks since lfo started
    pos = lfo_curtick % lfo_ticklen;                                      //calculate position within cycle in ticks
    index = (pos * LFO_LEN) / lfo_ticklen;                                //convert to position within wavetable array
  }
  else{
    pos = MIDI_Get_ClockTick() % lfo_clockticklen;                        //calculate position within cycle in clock ticks
    index = (pos * LFO_LEN) / lfo_clockticklen;                           //convert to position within wavetable array
  }
  lfo_level =  pgm_read_byte_near(&(lfo_store[index]));      //set lfo level
}
