/*
Copyright 2014 Paul Soulsby www.soulsbysynths.com
    This file is part of Atmegatron.

    Atmegadrum is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Atmegadrum is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Atmegadrum.  If not, see <http://www.gnu.org/licenses/>.
*/

#define MIDI_CHANNEL 9

#define NOTE_OFF 128
#define NOTE_ON 144
#define PITCH_WHEEL 224
#define CONTROLLER 176
#define MIDI_ACBASSDRUM 35
#define MIDI_BASSDRUM1 36
#define MIDI_STICK 37
#define MIDI_ACSNARE 38
#define MIDI_HANDCLAP 39
#define MIDI_ELECSNARE 40
#define MIDI_LOWFLOORTOM 41
#define MIDI_CLHIHAT 42
#define MIDI_HIGHFLOORTOM 43
#define MIDI_PEDHIHAT 44
#define MIDI_LOWTOM 45
#define MIDI_OPENHIHAT 46
#define MIDI_LOWMIDTOM 47
#define MIDI_HIMIDTOM 48
#define MIDI_CRASH 49
#define MIDI_COWBELL 56 

const byte MIDI_notemap[16] PROGMEM = {MIDI_BASSDRUM1,MIDI_ACBASSDRUM,MIDI_LOWFLOORTOM,MIDI_ACSNARE,MIDI_ELECSNARE,
	MIDI_HIGHFLOORTOM,MIDI_LOWTOM,MIDI_HANDCLAP,MIDI_LOWMIDTOM,MIDI_HIMIDTOM,MIDI_CLHIHAT,MIDI_PEDHIHAT,MIDI_OPENHIHAT,
MIDI_CRASH,MIDI_COWBELL,MIDI_STICK};

////local vars
byte incomingByte=0;
byte notebyte=0;
byte velocitybyte=0;
byte statusbuffer=0;
boolean firstbyte;



////lets and gets
boolean MIDI_clockpresent=false;
unsigned long MIDI_clocktick=0;

//
void MIDI_Init(){
	Serial.begin(31250);
}
//

void MIDI_NoteOn(byte notenum,byte velocity){
	byte i;
	for(i=0;i<16;i++){
		if(notenum==pgm_read_byte_near(&(MIDI_notemap[i]))){
			Seq_Let_Amplitude(i,7-(velocity>>4));
			master_index[i] = Wavestore_Get_Start(i);
		}
	}
}


//***********MIDI CLOCK*************

boolean MIDI_Get_ClockPresent(){
  return MIDI_clockpresent;
}
void MIDI_ClockStart(){
  MIDI_clockpresent = true; //start
  MIDI_clocktick = 0;
	Seq_Let_Playing(true,false);
}
void MIDI_ClockContinue(){
   MIDI_clockpresent = true; 
   Seq_Let_Playing(true,true);
	 
}
void MIDI_ClockStop(){
  MIDI_clockpresent = false; //stop
	Seq_Let_Playing(false,false);
}
void MIDI_ClockTick(){
  if (MIDI_clockpresent==true) {
    if (MIDI_clocktick%24 == 0){
      Hardware_LED_StartFlash(1,1);
    }
    MIDI_clocktick++;
  }
  
}
unsigned long MIDI_Get_ClockTick(){
  return MIDI_clocktick;
}

//*****MIDI POLL *********** 
void MIDI_Poll(){
  if (Serial.available() > 0) {
    do {
      // read the incoming byte:
      incomingByte = Serial.read();
      if (incomingByte>247) {
        // this is where MIDI clock stuff is done
        switch (incomingByte){
          case 248:
            MIDI_ClockTick();
            break;
          case 250:
            MIDI_ClockStart();
            break;
          case 251:
            MIDI_ClockStart();
            break;
          case 252:
            MIDI_ClockStop();
            break;
        }
      }
      else if (incomingByte>127) {
        statusbuffer = incomingByte;
        firstbyte = true;
        notebyte = 0;
        velocitybyte = 0;
      }
      else if (statusbuffer!=0) {
        if (firstbyte == true) {
          // must be first byte
          notebyte = incomingByte;
          firstbyte = false;
        }
        else {
          // so must be second byte then
          velocitybyte = incomingByte;
          //process the message here
          if (statusbuffer == (NOTE_ON | MIDI_CHANNEL) && velocitybyte != 0) {
            Hardware_LED_StartFlash(1,1);
            MIDI_NoteOn(notebyte,velocitybyte);
          }
          //now clear them for next note
          notebyte = 0;
          velocitybyte = 0; 
          firstbyte = true;         
        }
      }
    } while (Serial.available() > 0);
  } 
}

